-- Filename:     counter_bcd.vhd
-- Filetype:     VHDL Design File
-- Date:         9 mar 2014
-- Update:       -
-- Description:  A behavioral description of a BCD decade counter
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.2
-- Copyright:    (c)2013, De Haagse Hogeschool

-- The design file contains the description of a BCD decade
-- up counter with an enable and a asynchronous reset. The 
-- counter provides a terminal count signal for signalling that
-- the counter has reached its maximum value.

-- Use the IEEE libs...
library ieee;
use ieee.std_logic_1164.all;
-- Needed for '+' on unsigned vectors
use ieee.numeric_std.all;

-- The entity of the BCD decade counter
entity counter_bcd is
  port (clk    : in std_logic;
        areset : in std_logic;
        en     : in std_logic;
        q      : out unsigned(3 downto 0);
        tc     : out std_logic
       );
end entity counter_bcd;

-- The architecture
architecture rtl of counter_bcd is
-- Internal count value since port q is of out-mode
-- and you cannot read from an out-mode port.
signal count : unsigned (3 downto 0);

begin
  -- Process sensitive to clock and asynchronous reset
  process (clk, areset) is
  begin
    -- If reset active
    if areset = '1' then
      count <= "0000";
    -- On positive egde
    elsif rising_edge(clk) then
	   -- We're counting...
      if en = '1' then
		  -- If we reached count value 9, we're at maximum
		  -- so reset the counter value to 0. Otherwise,
		  -- we're not at the maximum value, so we increment
		  -- the count value.
        if count = "1001" then
          count <= "0000";
        else
          count <= count + 1;
        end if;
      end if;
    end if;
  end process;

  -- Signal if the counter has reached the maximum count
  tc <= '1' when count = "1001" and en = '1' else '0';

  -- Output the value of the counter
  q <= count;

  end architecture rtl;

