-- Filename:     tb_counter_bcd.vhd
-- Filetype:     VHDL Test Bench File
-- Date:         9 march 2014
-- Update:       -
-- Description:  A test bench for a bcd decade counter
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.1
-- Copyright:    (c)2014, De Haagse Hogeschool

-- The design file contains the description of a test bench
-- for a BCD decade up counter with an enable and a 
-- asynchronous clear. The counter provides a carry-out signal
-- for signalling that the counter has reached its maximum value.

-- Use library std_logic_1164
-- Use library numeric_std for addition
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tb_counter_bcd is
end entity tb_counter_bcd;

architecture sim of tb_counter_bcd is
-- The signals to be monitored
signal clk    : std_logic;
signal areset : std_logic;
signal en     : std_logic;
signal q      : unsigned(3 downto 0);
signal tc     : std_logic;

-- Component declaration of the counter
component counter_bcd is
    port (clk   : in std_logic;
			 areset : in std_logic;
          en     : in std_logic;
          q      : out unsigned(3 downto 0);
          tc     : out std_logic
         );
end component counter_bcd;

begin

	-- Instantiate the counter
	dut: counter_bcd
	port map (clk => clk, areset => areset, en => en, q => q, tc => tc);

	-- Generate a clock signal with a 20 ns period
	clockgen: process is
	begin
		clk <= '0';
		wait for 10 ns;
		clk <= '1';
		wait for 10 ns;
	end process clockgen;

	-- The data stimuli
	datagen: process is
	begin
		-- Disable and do not reset, wait a while...
		en <= '0';
		areset <= '0';
		wait until clk = '1';
		wait until clk = '1';
		wait for 1 ns;

		-- Force reset
		areset <= '1';
		wait until clk = '1';
		wait for 1 ns;

		-- Unforce reset, enable counting and
		-- wait until we've reached the maximum
		-- count value...
		en <= '1';
		areset <= '0';
		wait until q = "1001";
		wait until clk = '0';
		wait for 1 ns;

		-- Disable counting...
		en <= '0';
		wait until clk = '1';
		wait until clk = '1';
		wait for 1 ns;
		
		-- And enable again...
		en <= '1';
		wait;
	end process datagen;
	
end sim;
